#!/usr/bin/env python
import django
from django.conf import settings


settings.configure(
    DATABASES={"default": {"ENGINE": "django.db.backends.sqlite3"}},
    INSTALLED_APPS=[
        "django.contrib.contenttypes",
        "django.contrib.auth",
        "htpayway",
    ],
    MIDDLEWARE_CLASSES=[],
)
django.setup()


if __name__ == "__main__":
    from django.test.utils import get_runner

    test_runner = get_runner(settings)()
    result = test_runner.run_tests(["htpayway"])



from django.db import models
from django.conf import settings

import hashlib


STATUS = (
    ("created", "Created"),
    ("succeeded", "Succeeded"),
    ("failed", "Failed"),
)

CARD_TYPES = (
    ("1", "American Express"),
    ("2", "MasterCard"),
    ("3", "Visa"),
    ("4", "Diners"),
    ("5", "Maestro"),
)


class Transaction(models.Model):
    created_on = models.DateTimeField(auto_now_add=True)
    amount = models.DecimalField(max_digits=12, decimal_places=2)
    status = models.CharField(max_length=10, choices=STATUS)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
    )

    pgw_shop_id = models.CharField(max_length=8, blank=True)
    pgw_order_id = models.CharField(max_length=50)
    pgw_amount = models.CharField(max_length=12)
    pgw_authorization_type = models.CharField(max_length=1, blank=True)
    pgw_language = models.CharField(max_length=2, blank=True)
    pgw_success_url = models.CharField(max_length=1000)
    pgw_failure_url = models.CharField(max_length=1000)

    pgw_first_name = models.CharField(max_length=20, blank=True)
    pgw_last_name = models.CharField(max_length=20, blank=True)
    pgw_street = models.CharField(max_length=40, blank=True)
    pgw_city = models.CharField(max_length=20, blank=True)
    pgw_post_code = models.CharField(max_length=9, blank=True)
    pgw_country = models.CharField(max_length=50, blank=True)
    pgw_telephone = models.CharField(max_length=50, blank=True)
    pgw_email = models.CharField(max_length=50, blank=True)
    pgw_disable_installments = models.CharField(max_length=1, blank=True)

    pgw_signature = models.CharField(max_length=128, blank=True)

    pgw_result_code = models.CharField(max_length=4, blank=True)
    pgw_trace_ref = models.CharField(max_length=200, blank=True)
    pgw_transaction_id = models.CharField(max_length=10, blank=True)
    pgw_installments = models.CharField(max_length=10, blank=True)
    pgw_card_type_id = models.CharField(
        max_length=2, choices=CARD_TYPES, null=True, blank=True, default=None
    )

    response_received_on = models.DateTimeField(null=True, blank=True)
    response_signature_valid = models.BooleanField(default=False)

    class Meta:
        db_table = "htpayway_transactions"

    def __str__(self):
        tid = self.pgw_transaction_id
        fname = self.pgw_first_name
        lname = self.pgw_last_name
        return f"{tid} {fname} {lname} {self.user} {self.created_on} {self.amount:.2f}"

    def pgw_data(self):
        return {
            x.name: getattr(self, x.name)
            for x in self._meta.fields
            if x.name.startswith("pgw_")
        }

    def calc_outgoing_signature(self):
        """Calculates hash signature for outgoing request"""

        signature_string = "authorize-form" + self.pgw_secret_key
        fields = (
            "pgw_shop_id",
            "pgw_order_id",
            "pgw_amount",
            "pgw_authorization_type",
            "pgw_authorization_token",
            "pgw_language",
            "pgw_return_method",
            "pgw_success_url",
            "pgw_failure_url",
            "pgw_first_name",
            "pgw_last_name",
            "pgw_street",
            "pgw_city",
            "pgw_post_code",
            "pgw_country",
            "pgw_telephone",
            "pgw_email",
            "pgw_disable_installments",
        )

        for key in fields:
            value = getattr(self, key, None)
            if value is not None:
                signature_string += value
                signature_string += self.pgw_secret_key
        return hashlib.sha512(signature_string.encode("utf-8")).hexdigest()



from django.shortcuts import render, get_object_or_404
from django.views.decorators.csrf import csrf_exempt
from django.http import HttpResponse, Http404

from .models import Transaction
from .forms import SuccessForm, FailureForm, PaymentForm
from .utils import get_payway_class

import datetime


def begin(request, transaction_id):
    transaction = get_object_or_404(
        Transaction, pk=int(transaction_id), status="created"
    )
    payway = get_payway_class()(
        request=request, transaction=transaction, **transaction.pgw_data()
    )

    form = PaymentForm(initial=payway.pgw_data())
    return render(
        request, "htpayway/begin.html", {"form": form, "form_url": payway.pgw_form_url}
    )


@csrf_exempt
def success(request):
    if request.method == "POST":
        now = datetime.datetime.now()
        form = SuccessForm(request.POST)
        callback = False
        result = {}
        if form.is_valid():
            payway = get_payway_class()()
            pgw_order_id = form.cleaned_data["pgw_order_id"]
            transaction = Transaction.objects.get(
                pgw_order_id=pgw_order_id, status="created"
            )

            # extract data from response
            pgw_trace_ref = form.cleaned_data["pgw_trace_ref"]
            pgw_transaction_id = form.cleaned_data["pgw_transaction_id"]
            pgw_amount = form.cleaned_data["pgw_amount"]
            pgw_installments = form.cleaned_data["pgw_installments"]
            pgw_card_type_id = form.cleaned_data["pgw_card_type_id"]

            signature = payway.calc_incoming_signature(
                [
                    pgw_trace_ref,
                    pgw_transaction_id,
                    pgw_order_id,
                    pgw_amount,
                    pgw_installments,
                    pgw_card_type_id,
                ]
            )

            if signature == form.cleaned_data["pgw_signature"]:
                transaction.pgw_transaction_id = pgw_transaction_id
                transaction.pgw_trace_ref = pgw_trace_ref
                transaction.pgw_installments = pgw_installments
                transaction.pgw_card_type_id = pgw_card_type_id
                transaction.response_signature_valid = True
                transaction.status = "succeeded"

                callback = True
            else:
                transaction.response_signature_valid = False

            transaction.response_received_on = now
            transaction.save()

            if callback:
                result = payway.on_success(transaction)
                if isinstance(result, HttpResponse):
                    return result
            return render(request, "htpayway/success.html", result)
        raise Http404
    else:
        raise Http404


@csrf_exempt
def failure(request):
    if request.method == "POST":
        now = datetime.datetime.now()
        form = FailureForm(request.POST)
        callback = False
        result = {}
        if form.is_valid():
            payway = get_payway_class()()
            pgw_order_id = form.cleaned_data["pgw_order_id"]
            transaction = Transaction.objects.get(
                pgw_order_id=pgw_order_id, status="created"
            )

            # extract data from response
            pgw_result_code = form.cleaned_data["pgw_result_code"]
            pgw_trace_ref = form.cleaned_data["pgw_trace_ref"]
            pgw_signature = payway.calc_incoming_signature(
                [pgw_result_code, pgw_trace_ref, pgw_order_id]
            )

            if pgw_signature == form.cleaned_data["pgw_signature"]:
                transaction.pgw_result_code = pgw_result_code
                transaction.pgw_trace_ref = pgw_trace_ref
                callback = True
                transaction.response_signature_valid = True
                transaction.status = "failed"
            else:
                transaction.response_signature_valid = False

            transaction.pgw_signature = form.cleaned_data["pgw_signature"]
            transaction.response_received_on = now
            transaction.save()

            if callback:
                result = payway.on_failure(transaction)
                if isinstance(result, HttpResponse):
                    return result
        return render(request, "htpayway/failure.html", result)
    else:
        raise Http404



from django.test import TestCase, RequestFactory
from django.contrib.auth.models import AnonymousUser

from . import PayWay
from .utils import get_payway_class, begin_transaction, format_amount
from .models import Transaction

from decimal import Decimal, InvalidOperation


class CustomPayWay(PayWay):
    pgw_success_url = "http://localhost:8000/payway/success/"
    pgw_failure_url = "http://localhost:8000/payway/failure/"
    pgw_authorization_type = "0"
    pgw_language = "hr"

    @property
    def pgw_shop_id(self) -> str:
        return "123"

    @property
    def pgw_secret_key(self) -> str:
        return "secretkey"

    @property
    def pgw_form_url(self) -> str:
        return ""


class TestImports(TestCase):
    def test_missing_setting_raises(self):
        with self.assertRaises(ImportError):
            get_payway_class("")

    def test_get_payment_class_with_string(self):
        pw = get_payway_class("htpayway.tests.CustomPayWay")()
        self.assertEqual(pw.pgw_shop_id, "123")

    def test_get_payment_class_with_class(self):
        pw = get_payway_class(CustomPayWay)()
        self.assertEqual(pw.pgw_shop_id, "123")


class TestPayWay(TestCase):
    def setUp(self):
        self.payway = CustomPayWay()

    def test_create_signature_for_create(self):
        request = RequestFactory().get("/")
        request.user = AnonymousUser()
        transaction = begin_transaction(
            request,
            {"pgw_order_id": "1", "amount": "123.00"},
            htpayway_class=CustomPayWay,
        )

        self.assertEqual(
            transaction.pgw_signature,
            "fc424eb91bb260f8364326629b72de6ef7471cf4d09dc3c998657119cd0df2af"
            + "1f313c21108659e87573b0b6525c74f223b0378ab65dbf3e9ffd84697c31b319",
        )

    def test_pgw_arguments_are_initialized(self):
        p = CustomPayWay(pgw_email="a@a.com")
        self.assertEqual(p.pgw_email, "a@a.com")

    def test_non_pgw_arguments_are_skipped(self):
        p = CustomPayWay(foo=1)
        self.assertFalse(hasattr(p, "foo"))

    def test_pgw_data_from_model(self):
        p = Transaction(id=1, pgw_transaction_id=2, pgw_amount="300")
        pgw_data = p.pgw_data()

        self.assertNotIn("id", pgw_data)
        self.assertEqual(pgw_data["pgw_transaction_id"], 2)
        self.assertEqual(pgw_data["pgw_amount"], "300")

    def test_pgw_data_from_class(self):
        p = CustomPayWay(pgw_email="a@a.com")

        self.assertEqual(p.pgw_data()["pgw_email"], "a@a.com")


class TestUtils(TestCase):
    def test_format_amount_raises_on_non_decimal_input(self):
        with self.assertRaises(InvalidOperation):
            format_amount("")

        with self.assertRaises(TypeError):
            format_amount(None)

    def test_format_amount_with_2_decimal_places(self):
        self.assertEqual(format_amount("123.45"), "12345")

    def test_format_amount_with_3_decimal_places(self):
        self.assertEqual(format_amount(Decimal("1000.123")), "100012")



from django.http import HttpRequest
from django.urls import reverse
from django.shortcuts import render, get_object_or_404
from decimal import Decimal

from .models import Transaction
from .config import get_payway_class
from .forms import PaymentForm


def begin_transaction(request: HttpRequest, pgw_data, htpayway_class=None):
    """This should be called from your view
    to create Transaction object.
    """

    user = request.user if request.user.is_authenticated else None
    amount = pgw_data["amount"]

    tx = Transaction()
    tx.status = "created"
    tx.user = user

    """
    result.pgw_shop_id = pgw_data['pgw_shop_id']
    result.pgw_authorization_type = pgw_data['pgw_authorization_type']
    result.pgw_order_id = pgw_data['pgw_order_id']

    result.pgw_first_name = pgw_data.get('pgw_first_name', ''),
    result.pgw_last_name = pgw_data.get('pgw_last_name', ''),
    result.pgw_street = pgw_data.get('pgw_street', ''),
    result.pgw_city = pgw_data.get('pgw_city', ''),
    result.pgw_post_code = pgw_data.get('pgw_post_code', ''),
    result.pgw_country = pgw_data.get('pgw_country', ''),
    result.pgw_email = pgw_data.get('pgw_email', ''),
    """

    PayWayClass = get_payway_class(htpayway_class)()
    payway_data = PayWayClass.pgw_data()

    # every property that starts with `pgw_` should
    # be set as an attribute of a Transaction instance
    for x in payway_data:
        setattr(tx, x, payway_data[x])

    # and then we do the same for the `pgw_data` that was passed in
    for x in pgw_data:
        if x.startswith("pgw_"):
            setattr(tx, x, pgw_data[x])

    # setting success_url and failure_url if they were not passed in
    domain = request.get_host()
    protocol = request.scheme
    if tx.pgw_success_url is None:
        success_path = reverse("htpayway:success")
        tx.pgw_success_url = f"{protocol}://{domain}{success_path}"
    if tx.pgw_failure_url is None:
        failure_path = reverse("htpayway:failure")
        tx.pgw_failure_url = f"{protocol}://{domain}{failure_path}"

    # amount must be formatted according to the spec
    tx.amount = amount
    tx.pgw_amount = format_amount(amount)

    tx.pgw_signature = tx.calc_outgoing_signature()

    tx.save()
    return tx


def format_amount(amount):
    a = Decimal(amount).quantize(Decimal("0.01"))
    return str(a).replace(".", "")


def render_payway_form(request, transaction_id):
    transaction = get_object_or_404(
        Transaction, pk=int(transaction_id), status="created"
    )
    payway = get_payway_class()(
        request=request, transaction=transaction, **transaction.pgw_data()
    )

    form = PaymentForm(initial=payway.pgw_data())
    return render(
        request,
        "htpayway/payway-form.html",
        {"form": form, "form_url": payway.pgw_form_url},
    )



from django.urls import path
from . import views


app_name = "htpayway"
urlpatterns = [
    path("begin/<transaction_id>/", views.begin, name="begin"),
    path("success/", views.success, name="success"),
    path("failure/", views.failure, name="failure"),
]



# Generated by Django 5.2.3 on 2025-06-13 13:30

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Transaction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=12)),
                ('status', models.CharField(choices=[('created', 'Created'), ('succeeded', 'Succeeded'), ('failed', 'Failed')], max_length=10)),
                ('pgw_shop_id', models.CharField(blank=True, max_length=8)),
                ('pgw_order_id', models.CharField(max_length=50)),
                ('pgw_amount', models.CharField(max_length=12)),
                ('pgw_authorization_type', models.CharField(blank=True, max_length=1)),
                ('pgw_language', models.CharField(blank=True, max_length=2)),
                ('pgw_success_url', models.CharField(max_length=1000)),
                ('pgw_failure_url', models.CharField(max_length=1000)),
                ('pgw_first_name', models.CharField(blank=True, max_length=20)),
                ('pgw_last_name', models.CharField(blank=True, max_length=20)),
                ('pgw_street', models.CharField(blank=True, max_length=40)),
                ('pgw_city', models.CharField(blank=True, max_length=20)),
                ('pgw_post_code', models.CharField(blank=True, max_length=9)),
                ('pgw_country', models.CharField(blank=True, max_length=50)),
                ('pgw_telephone', models.CharField(blank=True, max_length=50)),
                ('pgw_email', models.CharField(blank=True, max_length=50)),
                ('pgw_disable_installments', models.CharField(blank=True, max_length=1)),
                ('pgw_signature', models.CharField(blank=True, max_length=128)),
                ('pgw_result_code', models.CharField(blank=True, max_length=4)),
                ('pgw_trace_ref', models.CharField(blank=True, max_length=200)),
                ('pgw_transaction_id', models.CharField(blank=True, max_length=10)),
                ('pgw_installments', models.CharField(blank=True, max_length=10)),
                ('pgw_card_type_id', models.CharField(blank=True, choices=[('1', 'American Express'), ('2', 'MasterCard'), ('3', 'Visa'), ('4', 'Diners'), ('5', 'Maestro')], default=None, max_length=2, null=True)),
                ('response_received_on', models.DateTimeField(blank=True, null=True)),
                ('response_signature_valid', models.BooleanField(default=False)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'htpayway_transactions',
            },
        ),
    ]



from django.contrib import admin
from htpayway.models import Transaction


@admin.register(Transaction)
class TransactionAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "pgw_name",
        "user",
        "created_on",
        "amount",
        "pgw_transaction_id",
        "status",
    )
    search_fields = [
        "pgw_first_name",
        "pgw_last_name",
        "pgw_email",
        "pgw_transaction_id",
    ]
    list_filter = [
        "created_on",
        "status",
    ]
    readonly_fields = ["created_on"]
    raw_id_fields = ["user"]

    fieldsets = [
        ("Order", {"fields": ["user", "created_on", "amount", "status"]}),
        (
            "Response",
            {
                "fields": [
                    ("response_received_on", "response_signature_valid"),
                    ("pgw_trace_ref", "pgw_transaction_id"),
                    ("pgw_installments", "pgw_card_type_id"),
                ]
            },
        ),
        (
            "Pgw data",
            {
                "fields": [
                    "pgw_shop_id",
                    "pgw_order_id",
                    "pgw_amount",
                    "pgw_authorization_type",
                    "pgw_first_name",
                    "pgw_last_name",
                    "pgw_street",
                    "pgw_city",
                    "pgw_post_code",
                    "pgw_country",
                    "pgw_email",
                ]
            },
        ),
    ]

    @admin.display(description="Name")
    def pgw_name(self, obj):
        return f"{obj.pgw_first_name} {obj.pgw_last_name}"



from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse
from django.utils.module_loading import import_string
from django.conf import settings

import hashlib


class PayWay:
    pgw_language = ""
    pgw_authorization_type = "1"
    pgw_disable_installments = "1"
    pgw_return_method = "POST"
    pgw_success_url = None
    pgw_failure_url = None

    def __init__(self, request=None, **pgw_data):
        skip = ["pgw_shop_id"]
        self.request = request
        for x in pgw_data:
            if x.startswith("pgw_"):
                if x in skip:
                    continue
                setattr(self, x, pgw_data[x])

    def pgw_data(self):
        return {x: getattr(self, x) for x in dir(self) if x.startswith("pgw_")}

    def calc_incoming_signature(self, data):
        """Calculates hash signature based on incoming request data"""
        signature_string = ""
        for item in data:
            signature_string += item
            signature_string += self.pgw_secret_key
        return hashlib.sha512(signature_string.encode("utf-8")).hexdigest()

    def on_success(self, transaction) -> HttpResponse | None:
        raise NotImplementedError

    def on_failure(self, transaction) -> HttpResponse | None:
        raise NotImplementedError

    @property
    def pgw_shop_id(self) -> str:
        return settings.HTPAYWAY_SHOP_ID

    @property
    def pgw_secret_key(self) -> str:
        return settings.HTPAYWAY_SECRET_KEY

    @property
    def pgw_form_url(self) -> str:
        return settings.HTPAYWAY_FORM_URL


def get_payway_class(htpayway_class=None):
    if htpayway_class is None:
        htpayway_class = settings.HTPAYWAY_CLASS

    if isinstance(htpayway_class, str):
        try:
            return import_string(htpayway_class)
        except AttributeError as e:
            raise ImproperlyConfigured(e)
    else:
        return htpayway_class
